#include "Complex.h"
#include <math.h>

Complex::Complex(double _re, double _im)
{
    re = _re;
    im = _im;
}

Complex::Complex(const Complex& rhs)
{
    re = rhs.re;
    im = rhs.im;
}

double  Complex::squareMag() const
{
    return re * re + im * im;
}

double  Complex::mag() const
{
    return sqrt(squareMag());
}

float Complex::arg() const
{
	return atan2(im, re);
}

Complex  Complex::conj() const
{
    return Complex(re, -im);
}

void    Complex::operator=(const Complex rhs)
{
    re = rhs.re;
    im = rhs.im;
}

Complex Complex::operator+(const Complex rhs) const
{
    Complex result;
    result.re = re + rhs.re;
    result.im = im + rhs.im;

    return result;
}

Complex Complex::operator-() const
{
    return Complex(-re, -im);
}

Complex Complex::operator-(const Complex rhs) const
{
    Complex result;
    result.re = re - rhs.re;
    result.im = im - rhs.im;

    return result;
}

Complex Complex::operator*(const double rhs) const
{
    Complex result;
    result.re = re * rhs;
    result.im = im * rhs;

    return result;
}

Complex Complex::operator*(const Complex rhs) const
{
    Complex result;
    result.re = re * rhs.re - im * rhs.im;
    result.im = im * rhs.re + re * rhs.im;

    return result;
}

Complex Complex::operator/(const double rhs) const
{
    Complex result;
    result.re = re / rhs;
    result.im = im / rhs;

    return result;
}

Complex Complex::operator/(const Complex rhs) const
{
    return (*this * rhs.conj()) / rhs.squareMag();
}

Complex Complex::Sin(const Complex z)
{
    Complex result;
    result.re = sin(z.re) * cosh(z.im);
    result.im = cos(z.re) * sinh(z.im);

    return result;
}

Complex Complex::Cos(const Complex z)
{
    Complex result;
    result.re = cos(z.re) * cosh(z.im);
    result.im = sin(z.re) * sinh(z.im);

    return result;
}
